package com.spring.gateway.filter;

import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.UUID;

import static com.spring.gateway.filter.CorrelationTrackingPostFilter.CORRELATION_ID;
import static com.spring.gateway.filter.FilterOrderType.PRE;

@Component
@Slf4j
public class CorrelationTrackingPreFilter implements GlobalFilter, Ordered {
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        log.info("Tracking filter invoked...");
        ServerHttpRequest request = exchange.getRequest();
        HttpHeaders headers = request.getHeaders();

        if (headers.containsKey(CORRELATION_ID)) {
            log.info("Tracked request with correlation id: {}", headers.get(CORRELATION_ID));
        } else {
            String newCorrelationId = UUID.randomUUID().toString();
            request = exchange.getRequest()
                    .mutate()
                    .header(CORRELATION_ID, newCorrelationId)
                    .build();
            log.info("Generated new correlation id: {}", newCorrelationId);
        }

        ServerWebExchange newExchange = exchange.mutate().request(request).build();
        return chain.filter(newExchange);
    }

    @Override
    public int getOrder() {
        return PRE.getValue();
    }
}
