package com.spring.gateway.filter;


import com.spring.gateway.error.UnauthorizedException;
import com.spring.gateway.model.User;
import com.spring.gateway.service.JwtUtil;
import com.spring.gateway.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.Optional;

@Slf4j
@Component
@RequiredArgsConstructor
public class JwtFilter implements GlobalFilter, Ordered {
    private final JwtUtil jwtUtil;
    private final UserService userService;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        final String uri = request.getURI().toString();
        if (uri.startsWith("/auth") || uri.contains("swagger") || uri.contains("api-docs")) {
            return chain.filter(exchange);
        }
        if (!request.getHeaders().containsKey("Authorization")) {
            throw new UnauthorizedException();
        }
        String token = request.getHeaders().getOrEmpty("Authorization").get(0).substring(7);
        try {
            if (jwtUtil.isExpired(token)) {
                throw new UnauthorizedException();
            }
        } catch (Exception ex) {
            throw new UnauthorizedException();
        }
        String email = jwtUtil.getSubject(token);
        Optional<User> user = userService.find(email);
        if (user.isPresent()) {
            return chain.filter(exchange);
        } else {
            throw new UnauthorizedException();
        }
    }

    @Override
    public int getOrder() {
        return 0;
    }
}