package com.spring.gateway.service;

import com.spring.gateway.dto.AuthRequest;
import com.spring.gateway.model.Role;
import com.spring.gateway.model.User;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class AuthUtil {
    private final UserService userService;
    private final JwtUtil jwtUtil;
    private final PasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public String signUp(AuthRequest req) {
        userService.create(
                req.getEmail(),
                passwordEncoder.encode(req.getPassword()),
                Role.USER
        );
        return "Success";
    }

    public String signIn(AuthRequest req) {
        final User user = userService.find(req.getEmail())
                .orElseThrow(() -> new RuntimeException("Incorrect email or password"));
        if (!passwordEncoder.matches(req.getPassword(), user.getPassword())) {
            throw new RuntimeException("Incorrect email or password");
        }
        return jwtUtil.generateToken(user);
    }
}
