package com.spring.minio.consumer;

import com.spring.minio.config.MinioAdapter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.KafkaHeaders;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.stereotype.Service;

@Slf4j
@Service
@RequiredArgsConstructor
public class Consumer {
    private final MinioAdapter minioAdapter;

    /**
     * Слушатель Kafka, который принимает сообщения из заданного топика и обрабатывает их.
     *
     * @param message Сообщение, полученное из Kafka.
     */
    @KafkaListener(topics = "test_topic", containerFactory = "kafkaListenerContainerFactory")
    public void listenGroupTopic(byte[] message, @Header(KafkaHeaders.RECEIVED_KEY) String key) {
        log.info("Received message with cor-id: " + key);
        try {
            minioAdapter.uploadFile("user1", key + " visitors_report.csv", message);

        } catch (Exception e) {
            log.error("Exception: ", e);
        }
    }
}
