package com.spring.report.kafka;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.concurrent.CompletableFuture;

@Slf4j
@Service
@RequiredArgsConstructor
public class Producer {
    private final KafkaTemplate<String, byte[]> kafkaTemplateByteArray;
    public static final String TEST_TOPIC = "test_topic";

    public void sendEvent(final String topic, final String key, final byte[] data) {
        Assert.hasText(topic, "topic must not be blank");
        Assert.notNull(data, "data not be null");

        CompletableFuture<SendResult<String, byte[]>> future = kafkaTemplateByteArray.send(topic, key, data);

        future.whenComplete((result, ex) -> {
            if (ex == null) {
                log.info("Kafka send complete");
            } else {
                log.error("Kafka fail send", ex);
            }
        });
    }
}
